"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = NextAuth;

var _crypto = require("crypto");

var _jwt = _interopRequireDefault(require("../lib/jwt"));

var _parseUrl = _interopRequireDefault(require("../lib/parse-url"));

var cookie = _interopRequireWildcard(require("./lib/cookie"));

var _callbackUrlHandler = _interopRequireDefault(require("./lib/callback-url-handler"));

var _providers = _interopRequireDefault(require("./lib/providers"));

var events = _interopRequireWildcard(require("./lib/events"));

var defaultCallbacks = _interopRequireWildcard(require("./lib/defaultCallbacks"));

var _providers2 = _interopRequireDefault(require("./routes/providers"));

var _signin = _interopRequireDefault(require("./routes/signin"));

var _signout = _interopRequireDefault(require("./routes/signout"));

var _callback = _interopRequireDefault(require("./routes/callback"));

var _session = _interopRequireDefault(require("./routes/session"));

var _pages = _interopRequireDefault(require("./pages"));

var _adapters = _interopRequireDefault(require("../adapters"));

var _logger = _interopRequireDefault(require("../lib/logger"));

var _redirect = _interopRequireDefault(require("./lib/redirect"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

if (!process.env.NEXTAUTH_URL) {
  _logger.default.warn('NEXTAUTH_URL', 'NEXTAUTH_URL environment variable not set');
}

function NextAuthHandler(_x, _x2, _x3) {
  return _NextAuthHandler.apply(this, arguments);
}

function _NextAuthHandler() {
  _NextAuthHandler = _asyncToGenerator(function* (req, res, userOptions) {
    return new Promise(function () {
      var _ref = _asyncToGenerator(function* (resolve) {
        var originalResEnd = res.end.bind(res);

        res.end = function () {
          resolve();
          return originalResEnd(...arguments);
        };

        res.redirect = (0, _redirect.default)(req, res);

        if (!req.query.nextauth) {
          var _error = 'Cannot find [...nextauth].js in pages/api/auth. Make sure the filename is written correctly.';

          _logger.default.error('MISSING_NEXTAUTH_API_ROUTE_ERROR', _error);

          res.status(500);
          return res.end("Error: ".concat(_error));
        }

        var {
          url,
          query,
          body
        } = req;
        var {
          nextauth,
          action = nextauth[0],
          provider = nextauth[1],
          error = nextauth[1]
        } = query;
        var {
          csrfToken: csrfTokenFromPost
        } = body;
        var {
          basePath,
          baseUrl
        } = (0, _parseUrl.default)(process.env.NEXTAUTH_URL || process.env.VERCEL_URL);
        var adapter;

        if (userOptions.adapter) {
          adapter = userOptions.adapter;
        } else if (userOptions.database) {
          adapter = _adapters.default.Default(userOptions.database);
        }

        var secret = userOptions.secret || (0, _crypto.createHash)('sha256').update(JSON.stringify(_objectSpread({
          baseUrl,
          basePath
        }, userOptions))).digest('hex');
        var useSecureCookies = userOptions.useSecureCookies || baseUrl.startsWith('https://');
        var cookiePrefix = useSecureCookies ? '__Secure-' : '';

        var cookies = _objectSpread({
          sessionToken: {
            name: "".concat(cookiePrefix, "next-auth.session-token"),
            options: {
              httpOnly: true,
              sameSite: 'lax',
              path: '/',
              secure: useSecureCookies
            }
          },
          callbackUrl: {
            name: "".concat(cookiePrefix, "next-auth.callback-url"),
            options: {
              sameSite: 'lax',
              path: '/',
              secure: useSecureCookies
            }
          },
          csrfToken: {
            name: "".concat(useSecureCookies ? '__Host-' : '', "next-auth.csrf-token"),
            options: {
              httpOnly: true,
              sameSite: 'lax',
              path: '/',
              secure: useSecureCookies
            }
          }
        }, userOptions.cookies);

        var sessionOptions = _objectSpread({
          jwt: false,
          maxAge: 30 * 24 * 60 * 60,
          updateAge: 24 * 60 * 60
        }, userOptions.session);

        var jwtOptions = _objectSpread({
          secret,
          maxAge: sessionOptions.maxAge,
          encode: _jwt.default.encode,
          decode: _jwt.default.decode
        }, userOptions.jwt);

        if (!adapter) {
          sessionOptions.jwt = true;
        }

        var eventsOptions = _objectSpread(_objectSpread({}, events), userOptions.events);

        var callbacksOptions = _objectSpread(_objectSpread({}, defaultCallbacks), userOptions.callbacks);

        var csrfToken;
        var csrfTokenVerified = false;

        if (req.cookies[cookies.csrfToken.name]) {
          var [csrfTokenValue, csrfTokenHash] = req.cookies[cookies.csrfToken.name].split('|');

          if (csrfTokenHash === (0, _crypto.createHash)('sha256').update("".concat(csrfTokenValue).concat(secret)).digest('hex')) {
            csrfToken = csrfTokenValue;

            if (req.method === 'POST' && csrfToken === csrfTokenFromPost) {
              csrfTokenVerified = true;
            }
          }
        }

        if (!csrfToken) {
          csrfToken = (0, _crypto.randomBytes)(32).toString('hex');
          var newCsrfTokenCookie = "".concat(csrfToken, "|").concat((0, _crypto.createHash)('sha256').update("".concat(csrfToken).concat(secret)).digest('hex'));
          cookie.set(res, cookies.csrfToken.name, newCsrfTokenCookie, cookies.csrfToken.options);
        }

        var options = _objectSpread(_objectSpread({
          debug: false,
          pages: {}
        }, userOptions), {}, {
          adapter,
          baseUrl,
          basePath,
          action,
          provider,
          cookies,
          secret,
          csrfToken,
          providers: (0, _providers.default)({
            providers: userOptions.providers,
            baseUrl,
            basePath
          }),
          session: sessionOptions,
          jwt: jwtOptions,
          events: eventsOptions,
          callbacks: callbacksOptions
        });

        req.options = options;

        if (options.debug) {
          process.env._NEXTAUTH_DEBUG = true;
        }

        var callbackUrl = yield (0, _callbackUrlHandler.default)(req, res);

        if (req.method === 'GET') {
          switch (action) {
            case 'providers':
              (0, _providers2.default)(req, res);
              break;

            case 'session':
              (0, _session.default)(req, res);
              break;

            case 'csrf':
              res.json({
                csrfToken
              });
              return res.end();

            case 'signin':
              if (options.pages.signIn) {
                var redirectUrl = "".concat(options.pages.signIn).concat(options.pages.signIn.includes('?') ? '&' : '?', "callbackUrl=").concat(callbackUrl);

                if (req.query.error) {
                  redirectUrl = "".concat(redirectUrl, "&error=").concat(req.query.error);
                }

                return res.redirect(redirectUrl);
              }

              (0, _pages.default)(req, res, 'signin', {
                providers: Object.values(options.providers),
                callbackUrl,
                csrfToken
              });
              break;

            case 'signout':
              if (options.pages.signOut) {
                return res.redirect("".concat(options.pages.signOut).concat(options.pages.signOut.includes('?') ? '&' : '?', "error=").concat(error));
              }

              (0, _pages.default)(req, res, 'signout', {
                csrfToken,
                callbackUrl
              });
              break;

            case 'callback':
              if (provider && options.providers[provider]) {
                (0, _callback.default)(req, res);
              } else {
                res.status(400);
                return res.end("Error: HTTP GET is not supported for ".concat(url));
              }

              break;

            case 'verify-request':
              if (options.pages.verifyRequest) {
                return res.redirect(options.pages.verifyRequest);
              }

              (0, _pages.default)(req, res, 'verify-request');
              break;

            case 'error':
              if (options.pages.error) {
                return res.redirect("".concat(options.pages.error).concat(options.pages.error.includes('?') ? '&' : '?', "error=").concat(error));
              }

              (0, _pages.default)(req, res, 'error', {
                error
              });
              break;

            default:
              res.status(404);
              return res.end();
          }
        } else if (req.method === 'POST') {
          switch (action) {
            case 'signin':
              if (!csrfTokenVerified) {
                return res.redirect("".concat(baseUrl).concat(basePath, "/signin?csrf=true"));
              }

              if (provider && options.providers[provider]) {
                (0, _signin.default)(req, res);
              }

              break;

            case 'signout':
              if (!csrfTokenVerified) {
                return res.redirect("".concat(baseUrl).concat(basePath, "/signout?csrf=true"));
              }

              (0, _signout.default)(req, res);
              break;

            case 'callback':
              if (provider && options.providers[provider]) {
                if (options.providers[provider].type === 'credentials' && !csrfTokenVerified) {
                  return res.redirect("".concat(baseUrl).concat(basePath, "/signin?csrf=true"));
                }

                (0, _callback.default)(req, res);
              } else {
                res.status(400);
                return res.end("Error: HTTP POST is not supported for ".concat(url));
              }

              break;

            default:
              res.status(400);
              return res.end("Error: HTTP POST is not supported for ".concat(url));
          }
        } else {
          res.status(400);
          return res.end("Error: HTTP ".concat(req.method, " is not supported for ").concat(url));
        }
      });

      return function (_x4) {
        return _ref.apply(this, arguments);
      };
    }());
  });
  return _NextAuthHandler.apply(this, arguments);
}

function NextAuth() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  if (args.length === 1) {
    return (req, res) => NextAuthHandler(req, res, args[0]);
  }

  return NextAuthHandler(...args);
}