"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = oAuthCallback;

var _crypto = require("crypto");

var _jsonwebtoken = require("jsonwebtoken");

var _client = _interopRequireDefault(require("./client"));

var _logger = _interopRequireDefault(require("../../../lib/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

class OAuthCallbackError extends Error {
  constructor(message) {
    super(message);
    this.name = 'OAuthCallbackError';
    this.message = message;
  }

}

function oAuthCallback(_x, _x2) {
  return _oAuthCallback.apply(this, arguments);
}

function _oAuthCallback() {
  _oAuthCallback = _asyncToGenerator(function* (req, csrfToken) {
    var _provider$version;

    var provider = req.options.providers[req.options.provider];
    var client = (0, _client.default)(provider);

    if ((_provider$version = provider.version) === null || _provider$version === void 0 ? void 0 : _provider$version.startsWith('2.')) {
      var {
        code,
        user,
        state
      } = req.query;

      if (provider.id !== 'apple') {
        var expectedState = (0, _crypto.createHash)('sha256').update(csrfToken).digest('hex');

        if (state !== expectedState) {
          throw new OAuthCallbackError('Invalid state returned from OAuth provider');
        }
      }

      if (req.method === 'POST') {
        try {
          var body = JSON.parse(JSON.stringify(req.body));

          if (body.error) {
            throw new Error(body.error);
          }

          code = body.code;
          user = body.user != null ? JSON.parse(body.user) : null;
        } catch (error) {
          _logger.default.error('OAUTH_CALLBACK_HANDLER_ERROR', error, req.body, provider.id, code);

          throw error;
        }
      }

      if (Object.prototype.hasOwnProperty.call(provider, 'useAuthTokenHeader')) {
        client.useAuthorizationHeaderforGET(provider.useAuthTokenHeader);
      } else {
        client.useAuthorizationHeaderforGET(true);
      }

      try {
        var {
          accessToken,
          refreshToken,
          results
        } = yield client.getOAuthAccessToken(code, provider);
        var tokens = {
          accessToken,
          refreshToken,
          idToken: results.id_token
        };
        var profileData;

        if (provider.idToken) {
          if (!(results === null || results === void 0 ? void 0 : results.id_token)) {
            throw new OAuthCallbackError();
          }

          profileData = decodeIdToken(results.id_token);
        } else {
          profileData = yield client.get(provider, accessToken, results);
        }

        return _getProfile({
          profileData,
          provider,
          tokens,
          user
        });
      } catch (error) {
        _logger.default.error('OAUTH_GET_ACCESS_TOKEN_ERROR', error, provider.id, code);

        throw error;
      }
    }

    try {
      var {
        oauth_token: oauthToken,
        oauth_verifier: oauthVerifier
      } = req.query;
      var {
        accessToken: _accessToken,
        refreshToken: _refreshToken,
        results: _results
      } = yield client.getOAuthAccessToken(oauthToken, null, oauthVerifier);

      var _profileData = yield client.get(provider.profileUrl, _accessToken, _refreshToken);

      var _tokens = {
        accessToken: _accessToken,
        refreshToken: _refreshToken,
        idToken: _results.id_token
      };
      return _getProfile({
        profileData: _profileData,
        tokens: _tokens,
        provider
      });
    } catch (error) {
      _logger.default.error('OAUTH_V1_GET_ACCESS_TOKEN_ERROR', error);

      throw error;
    }
  });
  return _oAuthCallback.apply(this, arguments);
}

function _getProfile(_x3) {
  return _getProfile2.apply(this, arguments);
}

function _getProfile2() {
  _getProfile2 = _asyncToGenerator(function* (_ref) {
    var {
      profileData,
      tokens: {
        accessToken,
        refreshToken,
        idToken
      },
      provider,
      user
    } = _ref;

    try {
      var _profile$email$toLowe, _profile$email;

      if (typeof profileData === 'string' || profileData instanceof String) {
        profileData = JSON.parse(profileData);
      }

      if (user != null) {
        profileData.user = user;
      }

      profileData.idToken = idToken;

      _logger.default.debug('PROFILE_DATA', profileData);

      var profile = yield provider.profile(profileData);
      return {
        profile: _objectSpread(_objectSpread({}, profile), {}, {
          email: (_profile$email$toLowe = (_profile$email = profile.email) === null || _profile$email === void 0 ? void 0 : _profile$email.toLowerCase()) !== null && _profile$email$toLowe !== void 0 ? _profile$email$toLowe : null
        }),
        account: {
          provider: provider.id,
          type: provider.type,
          id: profile.id,
          refreshToken,
          accessToken,
          accessTokenExpires: null
        },
        OAuthProfile: profileData
      };
    } catch (exception) {
      _logger.default.error('OAUTH_PARSE_PROFILE_ERROR', exception, profileData);

      return {
        profile: null,
        account: null,
        OAuthProfile: profileData
      };
    }
  });
  return _getProfile2.apply(this, arguments);
}

function decodeIdToken(idToken) {
  if (!idToken) {
    throw new OAuthCallbackError('Missing JWT ID Token');
  }

  return (0, _jsonwebtoken.decode)(idToken, {
    json: true
  });
}